@@ -1,4 +1,6 @@
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE OverloadedStrings #-}
+
 module Main where
 
 import Text.TeXMath
@@ -6,20 +8,22 @@ import Data.Char (isSpace)
 import Text.XML.Light
 import System.IO
 import System.Environment
-import Control.Applicative
 import System.Console.GetOpt
-import Data.List (intersperse)
 import System.Exit
 import Data.Maybe
 import Text.Pandoc.Definition
 import Network.URI (unEscapeString)
-import Data.List.Split (splitOn)
 import Data.Aeson (encode, (.=), object)
+import Data.Semigroup ((<>))
 import qualified Data.ByteString.Lazy as B
 import qualified Data.Text as T
+import qualified Data.Text.IO as T
 import Data.Version ( showVersion )
 import Paths_texmath (version)
 
+tshow :: Show a => a -> T.Text
+tshow = T.pack . show
+
 inHtml :: Element -> Element
 inHtml e =
   add_attr (Attr (unqual "xmlns") "http://www.w3.org/1999/xhtml") $
@@ -30,28 +34,28 @@ inHtml e =
         unode "meta" ()
     , unode "body" e ]
 
-type Reader = String -> Either String [Exp]
+type Reader = T.Text -> Either T.Text [Exp]
 data Writer = XMLWriter (DisplayType -> [Exp] -> Element)
-            | StringWriter (DisplayType -> [Exp] -> String)
+            | StringWriter (DisplayType -> [Exp] -> T.Text)
             | PandocWriter (DisplayType -> [Exp] -> Maybe [Inline])
 
-readers :: [(String, Reader)]
+readers :: [(T.Text, Reader)]
 readers = [
     ("tex", readTeX)
   , ("mathml", readMathML)
   , ("omml", readOMML)
   , ("native", readNative)]
 
-readNative :: String -> Either String [Exp]
+readNative :: T.Text -> Either T.Text [Exp]
 readNative s =
-  case reads s of
+  case reads (T.unpack s) of
        ((exps, ws):_) | all isSpace ws -> Right exps
        ((_, (_:_)):_) -> Left "Could not read whole input as native Exp"
        _ -> Left "Could not read input as native Exp"
 
-writers :: [(String, Writer)]
+writers :: [(T.Text, Writer)]
 writers = [
-    ("native", StringWriter (\_ es -> show es) )
+    ("native", StringWriter (\_ es -> tshow es) )
   , ("tex", StringWriter (\_ -> writeTeX))
   , ("eqn", StringWriter writeEqn)
   , ("omml",  XMLWriter writeOMML)
@@ -61,8 +65,8 @@ writers = [
 
 data Options = Options {
     optDisplay :: DisplayType
-  , optIn :: String
-  , optOut :: String }
+  , optIn :: T.Text
+  , optOut :: T.Text }
 
 def :: Options
 def = Options DisplayBlock "tex" "mathml"
@@ -73,48 +77,50 @@ options =
       (NoArg (\opts -> return opts {optDisplay = DisplayInline}))
       "Use the inline display style"
   , Option "f" ["from"]
-      (ReqArg (\s opts -> return opts {optIn = s}) "FORMAT")
-      ("Input format: " ++ (concat $ intersperse ", " (map fst readers)))
+      (ReqArg (\s opts -> return opts {optIn = T.pack s}) "FORMAT")
+      ("Input format: " <> T.unpack (T.intercalate ", " (map fst readers)))
   , Option "t" ["to"]
-      (ReqArg (\s opts -> return opts {optOut = s}) "FORMAT")
-      ("Output format: " ++ (concat $ intersperse ", " (map fst writers)))
+      (ReqArg (\s opts -> return opts {optOut = T.pack s}) "FORMAT")
+      ("Output format: " <> T.unpack (T.intercalate ", " (map fst writers)))
    , Option "v" ["version"]
       (NoArg (\_ -> do
-                      hPutStrLn stderr $ "Version " ++ showVersion version
+                      hPutStrLn stderr $ "Version " <> showVersion version
                       exitWith ExitSuccess))
       "Print version"
 
     , Option "h" ["help"]
         (NoArg (\_ -> do
                         prg <- getProgName
-                        hPutStrLn stderr (usageInfo (prg ++ " [OPTIONS] [FILE*]") options)
+                        hPutStrLn stderr (usageInfo (prg <> " [OPTIONS] [FILE*]") options)
                         exitWith ExitSuccess))
       "Show help"
   ]
 
-output :: DisplayType -> Writer -> [Exp] -> String
-output dt (XMLWriter w) es    = output dt (StringWriter (\dt' -> ppElement . w dt' )) es
+output :: DisplayType -> Writer -> [Exp] -> T.Text
+output dt (XMLWriter w) es    = output dt (StringWriter (\dt' -> T.pack . ppElement . w dt' )) es
 output dt (StringWriter w) es = w dt es
-output dt (PandocWriter w) es = show (fromMaybe fallback (w dt es))
-  where fallback = [Math mt (writeTeX es)]
+output dt (PandocWriter w) es = tshow (fromMaybe fallback (w dt es))
+  where fallback = [Math mt $ writeTeX es]
         mt = case dt of
                   DisplayBlock  -> DisplayMath
                   DisplayInline -> InlineMath
 
-err :: Bool -> Int -> String -> IO a
+err :: Bool -> Int -> T.Text -> IO a
 err cgi code msg = do
   if cgi
      then B.putStr $ encode $ object [T.pack "error" .= msg]
-     else hPutStrLn stderr msg
+     else T.hPutStrLn stderr msg
   exitWith $ ExitFailure code
 
-ensureFinalNewline :: String -> String
-ensureFinalNewline "" = ""
-ensureFinalNewline xs = if last xs == '\n' then xs else xs ++ "\n"
+ensureFinalNewline :: T.Text -> T.Text
+ensureFinalNewline xs = case T.unsnoc xs of
+  Nothing        -> xs
+  Just (_, '\n') -> xs
+  _              -> xs <> "\n"
 
-urlUnencode :: String -> String
-urlUnencode = unEscapeString . plusToSpace
-  where plusToSpace ('+':xs) = "%20" ++ plusToSpace xs
+urlUnencode :: T.Text -> T.Text
+urlUnencode = T.pack . unEscapeString . plusToSpace . T.unpack
+  where plusToSpace ('+':xs) = "%20" <> plusToSpace xs
         plusToSpace (x:xs)   = x : plusToSpace xs
         plusToSpace []       = []
 
@@ -132,8 +138,8 @@ runCommandLine = do
   let (actions, files, _) = getOpt RequireOrder options args
   opts <- foldl (>>=) (return def) actions
   inp <- case files of
-              [] -> getContents
-              _  -> concat <$> mapM readFile files
+              [] -> T.getContents
+              _  -> T.concat <$> mapM T.readFile files
   reader <- case lookup (optIn opts) readers of
                   Just r -> return r
                   Nothing -> err False 3 "Unrecognised reader"
@@ -142,17 +148,18 @@ runCommandLine = do
                   Nothing -> err False 5 "Unrecognised writer"
   case reader inp of
         Left msg -> err False 1 msg
-        Right v  -> putStr $ ensureFinalNewline
-                           $ output (optDisplay opts) writer v
+        Right v  -> T.putStr $ ensureFinalNewline
+                             $ output (optDisplay opts) writer v
   exitWith ExitSuccess
 
 runCGI :: IO ()
 runCGI = do
-  query <- getContents
-  let topairs xs = case break (=='=') xs of
-                        (ys,('=':zs)) -> (urlUnencode ys, urlUnencode zs)
-                        (ys,_)        -> (urlUnencode ys,"")
-  let pairs = map topairs $ splitOn "&" query
+  query <- T.getContents
+  let topairs xs = case T.break (=='=') xs of
+                     (ys, zs) -> case T.uncons zs of
+                       Just ('=', zs') -> (urlUnencode ys, urlUnencode zs')
+                       _               -> (urlUnencode ys, "")
+  let pairs = map topairs $ T.split (== '&') query
   inp <- case lookup "input" pairs of
                  Just x  -> return x
                  Nothing -> err True 3 "Query missing 'input'"
