@@ -1,4 +1,5 @@
 {-# LANGUAGE PatternGuards #-}
+{-# LANGUAGE OverloadedStrings #-}
 
 {-
 Copyright (C) 2014 Jesse Rosenthal <jrosenthal@jhu.edu>
@@ -36,13 +37,14 @@ import Text.XML.Light
 import Data.Maybe (isJust, mapMaybe, fromMaybe)
 import Data.List (intercalate)
 import Data.Char (isDigit, readLitChar)
+import qualified Data.Text as T
 import Text.TeXMath.Types
 import Text.TeXMath.Shared (fixTree, getSpaceWidth, getOperator)
 import Text.TeXMath.Unicode.ToTeX (getSymbolType)
 import Control.Applicative ((<$>))
-import Text.TeXMath.Unicode.Fonts (getUnicode, stringToFont)
+import Text.TeXMath.Unicode.Fonts (getUnicode, textToFont)
 
-readOMML :: String -> Either String [Exp]
+readOMML :: T.Text -> Either T.Text [Exp]
 readOMML s | Just e <- parseXMLDoc s =
   case elemToOMML e of
     Just exs -> Right $ map fixTree $ unGroup exs
@@ -71,6 +73,7 @@ unwrapWTags elements = concatMap unwrapChild elements
                                 Just "w" -> elChildren element
                                 _        -> [element]
 
+-- Kept as String because of Text.XML.Light
 isElem :: String -> String -> Element -> Bool
 isElem prefix name element =
   let qp = fromMaybe "" (qPrefix (elName element))
@@ -78,14 +81,15 @@ isElem prefix name element =
    qName (elName element) == name &&
    qp == prefix
 
-hasElemName:: String -> String -> QName -> Bool
+-- Kept as String because of Text.XML.Light
+hasElemName :: String -> String -> QName -> Bool
 hasElemName prefix name qn =
   let qp = fromMaybe "" (qPrefix qn)
   in
    qName qn == name &&
    qp       == prefix
 
-data OMathRunElem = TextRun String
+data OMathRunElem = TextRun T.Text
                   | LnBrk
                   | Tab
                     deriving Show
@@ -128,8 +132,8 @@ elemToBases _ = Nothing
 -- text lines into multiple columns. That's tricky, though, and this
 -- will get us most of the way for the time being.
 filterAmpersand :: Exp -> Exp
-filterAmpersand (EIdentifier s)   = EIdentifier (filter ('&' /=) s)
-filterAmpersand (EText tt s)      = EText tt (filter ('&' /=) s)
+filterAmpersand (EIdentifier s)   = EIdentifier (T.filter ('&' /=) s)
+filterAmpersand (EText tt s)      = EText tt (T.filter ('&' /=) s)
 filterAmpersand (EStyled tt exps) = EStyled tt (map filterAmpersand exps)
 filterAmpersand (EGrouped exps)   = EGrouped (map filterAmpersand exps)
 filterAmpersand e                    = e
@@ -171,7 +175,7 @@ elemToOMathRunElem :: Element -> Maybe OMathRunElem
 elemToOMathRunElem element
   | isElem "w" "t" element
     || isElem "m" "t" element
-    || isElem "w" "delText" element = Just $ TextRun $ strContent element
+    || isElem "w" "delText" element = Just $ TextRun $ T.pack $ strContent element
   | isElem "w" "br" element = Just LnBrk
   | isElem "w" "tab" element = Just Tab
   | isElem "w" "sym" element = Just $ TextRun $ getSymChar element
@@ -186,13 +190,13 @@ elemToOMathRunElems _ = Nothing
 
 ----- And now the TeXMath Creation
 
-oMathRunElemToString :: OMathRunElem -> String
-oMathRunElemToString (TextRun s) = s
-oMathRunElemToString (LnBrk) = ['\n']
-oMathRunElemToString (Tab) = ['\t']
+oMathRunElemToText :: OMathRunElem -> T.Text
+oMathRunElemToText (TextRun s) = s
+oMathRunElemToText (LnBrk) = "\n"
+oMathRunElemToText (Tab) = "\t"
 
-oMathRunElemsToString :: [OMathRunElem] -> String
-oMathRunElemsToString = concatMap oMathRunElemToString
+oMathRunElemsToText :: [OMathRunElem] -> T.Text
+oMathRunElemsToText = T.concat . map oMathRunElemToText
 
 oMathRunTextStyleToTextType :: OMathRunTextStyle -> Maybe TextType
 oMathRunTextStyleToTextType (Normal) = Just $ TextNormal
@@ -241,11 +245,11 @@ elemToExps' element | isElem "m" "acc" element = do
             findAttrBy (hasElemName "m" "val") >>=
             Just . head
       chr' = case chr of
-        Just c -> c
-        Nothing -> '\x302'       -- default to wide hat.
+        Just c -> T.singleton c
+        Nothing -> "\x302"       -- default to wide hat.
   baseExp <- filterChildName (hasElemName "m" "e") element >>=
              elemToBase
-  return $ [EOver False baseExp (ESymbol Accent [chr'])]
+  return $ [EOver False baseExp (ESymbol Accent chr')]
 elemToExps' element | isElem "m" "bar" element = do
   pos <- filterChildName (hasElemName "m" "barPr") element >>=
             filterChildName (hasElemName "m" "pos") >>=
@@ -282,12 +286,12 @@ elemToExps' element | isElem "m" "d" element =
                filterChildName (hasElemName "m" "endChr") >>=
                findAttrBy (hasElemName "m" "val") >>=
                (\c -> if null c then (Just ' ') else (Just $ head c))
-      beg = fromMaybe '(' begChr
-      end = fromMaybe ')' endChr
-      sep = fromMaybe '|' sepChr
-      exps = intercalate [Left [sep]] inDelimExps
+      beg = maybe "(" T.singleton begChr
+      end = maybe ")" T.singleton endChr
+      sep = maybe "|" T.singleton sepChr
+      exps = intercalate [Left sep] inDelimExps
   in
-   Just [EDelimited [beg] [end] exps]
+   Just [EDelimited beg end exps]
 elemToExps' element | isElem "m" "eqArr" element =
   let expLst = mapMaybe elemToBases (elChildren element)
       expLst' = map (\es -> [map filterAmpersand es]) expLst
@@ -315,22 +319,30 @@ elemToExps' element | isElem "m" "groupChr" element = do
       pos = gPr >>=
             filterChildName (hasElemName "m" "pos") >>=
             findAttrBy (hasElemName "m" "val")
+      justif = gPr >>=
+               filterChildName (hasElemName "m" "vertJC") >>=
+               findAttrBy (hasElemName "m" "val")
   baseExp <- filterChildName (hasElemName "m" "e") element >>=
              elemToBase
   case pos of
     Just "top" ->
       let chr' = case chr of
-            Just (c:_) -> c
-            _           -> '\65079'   -- default to overbrace
+            Just (c:_) -> T.singleton c
+            _           -> "\65079"   -- default to overbrace
       in
-       return [EOver False baseExp (ESymbol TOver [chr'])]
-    Just "bot" ->
+       return $
+         case justif of
+           Just "top" -> [EUnder False (ESymbol TOver chr') baseExp]
+           _ -> [EOver False baseExp (ESymbol TOver chr')]
+    _ -> -- bot is default
       let chr' = case chr of
-            Just (c:_) -> c
-            _           -> '\65080'   -- default to underbrace
+            Just (c:_) -> T.singleton c
+            _           -> "\65080"   -- default to underbrace
       in
-       return [EUnder False baseExp (ESymbol TUnder [chr'])]
-    _          -> Nothing
+       return $
+         case justif of
+           Just "top" -> [EUnder False baseExp (ESymbol TUnder chr')]
+           _ -> [EOver False (ESymbol TUnder chr') baseExp]
 elemToExps' element | isElem "m" "limLow" element = do
   baseExp <- filterChildName (hasElemName "m" "e") element
           >>= elemToBase
@@ -360,8 +372,8 @@ elemToExps' element | isElem "m" "nary" element = do
                 filterChildName (hasElemName "m" "chr") >>=
                 findAttrBy (hasElemName "m" "val")
       opChr = case naryChr of
-        Just (c:_) -> c
-        _          -> '\8747'   -- default to integral
+        Just (c:_) -> T.singleton c
+        _          -> "\8747"   -- default to integral
       limLoc = naryPr >>=
                filterChildName (hasElemName "m" "limLoc") >>=
                findAttrBy (hasElemName "m" "val")
@@ -373,12 +385,12 @@ elemToExps' element | isElem "m" "nary" element = do
              elemToBase
   case limLoc of
     Just "undOvr" -> return [EUnderover True
-                              (ESymbol Op [opChr])
+                              (ESymbol Op opChr)
                               (EGrouped subExps)
                               (EGrouped supExps)
                             , baseExp]
     _             -> return [ESubsup
-                              (ESymbol Op [opChr])
+                              (ESymbol Op opChr)
                               (EGrouped subExps)
                               (EGrouped supExps)
                             , baseExp]
@@ -429,39 +441,41 @@ elemToExps' element | isElem "m" "sSup" element = do
   return [ESuper baseExp (EGrouped supExps)]
 elemToExps' element | isElem "m" "r" element = do
   let mrPr = filterChildName (hasElemName "m" "rPr") element
-      lit = mrPr >>=
-            filterChildName (hasElemName "m" "lit") >>=
-            findAttrBy (hasElemName "m" "val")
-      txtSty = elemToOMathRunTextStyle element
+      lit = mrPr >>= filterChildName (hasElemName "m" "lit")
+      nor = mrPr >>= filterChildName (hasElemName "m" "nor")
+      txtSty = oMathRunTextStyleToTextType $ elemToOMathRunTextStyle element
   mrElems <- elemToOMathRunElems element
-  return $ case oMathRunTextStyleToTextType txtSty of
-    Nothing -> interpretString $ oMathRunElemsToString mrElems
-    Just textType ->
-      case lit of
-        Just "on" ->
-          [EText textType (oMathRunElemsToString mrElems)]
-        _         ->
-          [EStyled textType $ interpretString $ oMathRunElemsToString mrElems]
+  return $
+    if null lit && null nor
+       then case txtSty of
+              Nothing ->
+                interpretText $ oMathRunElemsToText mrElems
+              Just textSty ->
+                [EStyled textSty $ interpretText $ oMathRunElemsToText mrElems]
+       else [EText (fromMaybe TextNormal txtSty) $ oMathRunElemsToText mrElems]
 elemToExps' _ = Nothing
 
 interpretChar :: Char -> Exp
-interpretChar c | isDigit c = ENumber [c]
+interpretChar c | isDigit c = ENumber $ T.singleton c
 interpretChar c = case getSymbolType c of
-  Alpha           -> EIdentifier [c]
-  Ord | isDigit c -> ENumber [c]
+  Alpha           -> EIdentifier c'
+  Ord | isDigit c -> ENumber c'
       | otherwise -> case getSpaceWidth c of
                            Just x  -> ESpace x
-                           Nothing -> ESymbol Ord [c]
-  symType         -> ESymbol symType [c]
+                           Nothing -> ESymbol Ord c'
+  symType         -> ESymbol symType c'
+  where
+    c' = T.singleton c
 
-interpretString :: String -> [Exp]
-interpretString [c]       = [interpretChar c]
-interpretString s
-  | all isDigit s         = [ENumber s]
+interpretText :: T.Text -> [Exp]
+interpretText s
+  | Just (c, xs) <- T.uncons s
+  , T.null xs = [interpretChar c]
+  | T.all isDigit s         = [ENumber s]
   | isJust (getOperator (EMathOperator s))
                           = [EMathOperator s]
   | otherwise             =
-      case map interpretChar s of
+      case map interpretChar (T.unpack s) of
             xs | all isIdentifierOrSpace xs -> [EText TextNormal s]
                | otherwise                  -> xs
   where isIdentifierOrSpace (EIdentifier _) = True
@@ -469,15 +483,15 @@ interpretString s
         isIdentifierOrSpace _               = False
 
 -- The char attribute is a hex string
-getSymChar :: Element -> String
+getSymChar :: Element -> T.Text
 getSymChar element
   | Just s <- lowerFromPrivate <$> getCodepoint
   , Just font <- getFont =
   let [(char, _)] = readLitChar ("\\x" ++ s) in
-    maybe "" (:[]) $ getUnicode font char
+    maybe "" T.singleton $ getUnicode font char
   where
     getCodepoint = findAttrBy (hasElemName "w" "char") element
-    getFont = stringToFont =<< findAttrBy (hasElemName "w" "font") element
+    getFont = (textToFont . T.pack) =<< findAttrBy (hasElemName "w" "font") element
     lowerFromPrivate ('F':xs) = '0':xs
     lowerFromPrivate xs = xs
 getSymChar _ = ""
