@@ -1,4 +1,4 @@
-{-# LANGUAGE GeneralizedNewtypeDeriving, ViewPatterns, GADTs #-}
+{-# LANGUAGE GeneralizedNewtypeDeriving, ViewPatterns, GADTs, OverloadedStrings #-}
 {-
 Copyright (C) 2014 Matthew Pickering <matthewtpickering@gmail.com>
 
@@ -20,41 +20,43 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 module Text.TeXMath.Writers.Eqn (writeEqn) where
 
-import Data.List (intercalate, transpose)
+import Data.List (transpose)
 import Data.Char (isAscii, ord)
+import qualified Data.Text as T
 import Text.Printf (printf)
 import Text.TeXMath.Types
 import qualified Text.TeXMath.Shared as S
 import Data.Generics (everywhere, mkT)
 import Data.Ratio ((%))
+import Data.Semigroup ((<>))
 
 -- import Debug.Trace
 -- tr' x = trace (show x) x
 
 -- | Transforms an expression tree to equivalent Eqn with the default
 -- packages (amsmath and amssymb)
-writeEqn :: DisplayType -> [Exp] -> String
+writeEqn :: DisplayType -> [Exp] -> T.Text
 writeEqn dt exprs =
-  intercalate " " $ map writeExp $ everywhere (mkT $ S.handleDownup dt) exprs
+  T.intercalate " " $ map writeExp $ everywhere (mkT $ S.handleDownup dt) exprs
 
 -- like writeExp but inserts {} if contents contain a space
-writeExp' :: Exp -> String
+writeExp' :: Exp -> T.Text
 writeExp' e@(EGrouped _) = writeExp e
-writeExp' e = if ' ' `elem` s
-                 then "{" ++ s ++ "}"
+writeExp' e = if T.any (== ' ') s
+                 then "{" <> s <> "}"
                  else s
                where s = writeExp e
 
-writeExps :: [Exp] -> String
-writeExps = intercalate " " . map writeExp
+writeExps :: [Exp] -> T.Text
+writeExps = T.intercalate " " . map writeExp
 
-writeExp :: Exp -> String
+writeExp :: Exp -> T.Text
 writeExp (ENumber s) = s
-writeExp (EGrouped es) = "{" ++ writeExps es ++ "}"
+writeExp (EGrouped es) = "{" <> writeExps es <> "}"
 writeExp (EDelimited open close es) =
-  "left " ++ mbQuote open ++ " " ++ intercalate " " (map fromDelimited es) ++
-  " right " ++ mbQuote close
-  where fromDelimited (Left e)  = "\"" ++ e ++ "\""
+  "left " <> mbQuote open <> " " <> T.intercalate " " (map fromDelimited es) <>
+  " right " <> mbQuote close
+  where fromDelimited (Left e)  = "\"" <> e <> "\""
         fromDelimited (Right e) = writeExp e
         mbQuote "" = "\"\""
         mbQuote s  = s
@@ -63,8 +65,8 @@ writeExp (EMathOperator s) =
                "tanh", "arc", "max", "min", "lim",
                "log", "ln", "exp"]
      then s
-     else "\"" ++ s ++ "\""
-writeExp (ESymbol Ord [c])  -- do not render "invisible operators"
+     else "\"" <> s <> "\""
+writeExp (ESymbol Ord (T.unpack -> [c]))  -- do not render "invisible operators"
   | c `elem` ['\x2061'..'\x2064'] = "" -- see 3.2.5.5 of mathml spec
 writeExp (EIdentifier s) = writeExp (ESymbol Ord s)
 writeExp (ESymbol t s) =
@@ -129,19 +131,19 @@ writeExp (ESymbol t s) =
     "\958" -> "xi"
     "\926" -> "XI"
     "\950" -> "zeta"
-    _      -> let s' = if all isAscii s
+    _      -> let s' = if T.all isAscii s
                           then s
-                          else "\\[" ++ unwords (map toUchar s) ++ "]"
-                  toUchar c = printf "u%04X" (ord c)
-              in  if length s > 1 && (t == Rel || t == Bin || t == Op)
-                     then "roman{\"" ++
+                          else "\\[" <> T.unwords (map toUchar $ T.unpack s) <> "]"
+                  toUchar c = T.pack $ printf "u%04X" (ord c)
+              in  if T.length s > 1 && (t == Rel || t == Bin || t == Op)
+                     then "roman{\"" <>
                           (if t == Rel || t == Bin
                               then " "
-                              else "") ++
-                          s' ++
+                              else "") <>
+                          s' <>
                           (if t == Rel || t == Bin || t == Op
                               then " "
-                              else "") ++
+                              else "") <>
                           "\"}"
                      else s'
 
@@ -149,56 +151,58 @@ writeExp (ESpace d) =
   case d of
       _ | d > 0 && d < (2 % 9) -> "^"
         | d >= (2 % 9) && d < (3 % 9) -> "~"
-        | d < 0     -> "back " ++ show (floor (-1 * d * 100) :: Int)
-        | otherwise -> "fwd " ++ show (floor (d * 100) :: Int)
-writeExp (EFraction fractype e1 e2) = writeExp' e1 ++ op ++ writeExp' e2
+        | d < 0     -> "back " <> tshow (floor (-1 * d * 100) :: Int)
+        | otherwise -> "fwd " <> tshow (floor (d * 100) :: Int)
+writeExp (EFraction fractype e1 e2) = writeExp' e1 <> op <> writeExp' e2
   where op = if fractype == NoLineFrac
                 then " / "
                 else " over "
-writeExp (ESub b e1) = writeExp' b ++ " sub " ++ writeExp' e1
-writeExp (ESuper b e1) = writeExp' b ++ " sup " ++ writeExp' e1
+writeExp (ESub b e1) = writeExp' b <> " sub " <> writeExp' e1
+writeExp (ESuper b e1) = writeExp' b <> " sup " <> writeExp' e1
 writeExp (ESubsup b e1 e2) =
-  writeExp' b ++ " sub " ++ writeExp' e1 ++ " sup " ++ writeExp' e2
+  writeExp' b <> " sub " <> writeExp' e1 <> " sup " <> writeExp' e2
 writeExp (EOver _convertible b e1) =
-  writeExp' b ++ " to " ++ writeExp' e1
+  writeExp' b <> " to " <> writeExp' e1
 writeExp (EUnder _convertible b e1) =
-  writeExp' b ++ " from " ++ writeExp' e1
+  writeExp' b <> " from " <> writeExp' e1
 writeExp (EUnderover convertible b e1@(ESymbol Accent _) e2) =
   writeExp (EUnder convertible (EOver False b e2) e1)
 writeExp (EUnderover convertible b e1 e2@(ESymbol Accent _)) =
   writeExp (EOver convertible (EUnder False b e1) e2)
 writeExp (EUnderover _convertible b e1 e2) =
-  writeExp' b ++ " from " ++ writeExp' e1 ++ " to " ++ writeExp' e2
-writeExp (ESqrt e) = "sqrt " ++ writeExp' e
-writeExp (ERoot i e) = "\"\" sup " ++ writeExp' i ++ " sqrt " ++ writeExp' e
-writeExp (EPhantom e) = "hphantom " ++ writeExp' e
+  writeExp' b <> " from " <> writeExp' e1 <> " to " <> writeExp' e2
+writeExp (ESqrt e) = "sqrt " <> writeExp' e
+writeExp (ERoot i e) = "\"\" sup " <> writeExp' i <> " sqrt " <> writeExp' e
+writeExp (EPhantom e) = "hphantom " <> writeExp' e
 writeExp (EBoxed e) = writeExp e -- TODO: any way to do this?
 writeExp (EScaled _size e) = writeExp e -- TODO: any way?
 writeExp (EText ttype s) =
-  let quoted = "\"" ++ s ++ "\""
+  let quoted = "\"" <> s <> "\""
   in case ttype of
-       TextNormal -> "roman " ++ quoted
+       TextNormal -> "roman " <> quoted
        TextItalic -> quoted
-       TextBold   -> "bold " ++ quoted
-       TextBoldItalic -> "bold italic " ++ quoted
+       TextBold   -> "bold " <> quoted
+       TextBoldItalic -> "bold italic " <> quoted
        _   -> quoted
 writeExp (EStyled ttype es) =
-  let contents = "{" ++ writeExps es ++ "}"
+  let contents = "{" <> writeExps es <> "}"
   in case ttype of
-       TextNormal -> "roman " ++ contents
-       TextItalic -> "italic " ++ contents
-       TextBold   -> "bold " ++ contents
-       TextBoldItalic -> "bold italic " ++ contents
+       TextNormal -> "roman " <> contents
+       TextItalic -> "italic " <> contents
+       TextBold   -> "bold " <> contents
+       TextBoldItalic -> "bold italic " <> contents
        _   -> contents
 writeExp (EArray aligns rows) =
-  "matrix{\n" ++ concat cols ++ "}"
+  "matrix{\n" <> T.concat cols <> "}"
   where cols = zipWith tocol aligns (transpose rows)
         tocol al cs =
           (case al of
                AlignLeft -> "lcol"
                AlignCenter -> "ccol"
-               AlignRight -> "rcol") ++
-            "{ " ++ intercalate " above " (map tocell cs) ++ " }\n"
+               AlignRight -> "rcol") <>
+            "{ " <> T.intercalate " above " (map tocell cs) <> " }\n"
         tocell [e] = writeExp' e
         tocell es  = writeExp (EGrouped es)
 
+tshow :: Show a => a -> T.Text
+tshow = T.pack . show
