@@ -1,4 +1,4 @@
-{-# LANGUAGE ViewPatterns, ScopedTypeVariables #-}
+{-# LANGUAGE ViewPatterns, ScopedTypeVariables, OverloadedStrings #-}
 {-
 Copyright (C) 2009 John MacFarlane <jgm@berkeley.edu>
 
@@ -30,6 +30,8 @@ import Data.Generics (everywhere, mkT)
 import Text.TeXMath.Shared (getMMLType, handleDownup)
 import Text.TeXMath.Readers.MathML.MMLDict (getMathMLOperator)
 import Control.Applicative ((<$>))
+import Data.Semigroup ((<>))
+import qualified Data.Text as T
 import Text.Printf
 
 -- | Transforms an expression tree to a MathML XML tree
@@ -64,27 +66,30 @@ showFraction tt ft x y =
 spaceWidth :: Rational -> Element
 spaceWidth w =
   withAttribute "width" (dropTrailing0s
-     (printf "%.3f" (fromRational w :: Double)) ++ "em") $ unode "mspace" ()
+     (T.pack $ printf "%.3f" (fromRational w :: Double)) <> "em") $ unode "mspace" ()
 
 makeStretchy :: FormType -> Element -> Element
 makeStretchy (fromForm -> t)  = withAttribute "stretchy" "true"
                                 . withAttribute "form" t
 
-fromForm :: FormType -> String
+fromForm :: FormType -> T.Text
 fromForm FInfix   = "infix"
 fromForm FPostfix = "postfix"
 fromForm FPrefix  = "prefix"
 
-
 makeScaled :: Rational -> Element -> Element
 makeScaled x = withAttribute "minsize" s . withAttribute "maxsize" s
-  where s = dropTrailing0s $ printf "%.3f" (fromRational x :: Double)
+  where s = dropTrailing0s $ T.pack $ printf "%.3f" (fromRational x :: Double)
 
-dropTrailing0s :: String -> String
-dropTrailing0s = reverse . go . reverse
-  where go ('0':'.':xs) = '0':'.':xs
-        go ('0':xs) = go xs
-        go xs       = xs
+
+dropTrailing0s :: T.Text -> T.Text
+dropTrailing0s t = case T.unsnoc t of -- T.spanEnd does not exist
+  Just (ts, '0') -> addZero $ T.dropWhileEnd (== '0') ts
+  _              -> t
+  where
+    addZero x = case T.unsnoc x of
+      Just (_, '.') -> T.snoc x '0'
+      _ -> x
 
 makeStyled :: TextType -> [Element] -> Element
 makeStyled a es = withAttribute "mathvariant" attr
@@ -92,16 +97,20 @@ makeStyled a es = withAttribute "mathvariant" attr
   where attr = getMMLType a
 
 -- Note: Converts strings to unicode directly, as few renderers support those mathvariants.
-makeText :: TextType -> String -> Element
+makeText :: TextType -> T.Text -> Element
 makeText a s = case (leadingSp, trailingSp) of
                    (False, False) -> s'
                    (True,  False) -> mrow [sp, s']
                    (False, True)  -> mrow [s', sp]
                    (True,  True)  -> mrow [sp, s', sp]
   where sp = spaceWidth (1/3)
-        s' = withAttribute "mathvariant" attr $ unode "mtext" $ toUnicode a s
-        trailingSp = not (null s) && last s `elem` " \t"
-        leadingSp  = not (null s) && head s `elem` " \t"
+        s' = withAttribute "mathvariant" attr $ tunode "mtext" $ toUnicode a s
+        trailingSp = case T.unsnoc s of
+          Just (_, c) -> T.any (== c) " \t"
+          _           -> False
+        leadingSp  = case T.uncons s of
+          Just (c, _) -> T.any (== c) " \t"
+          _           -> False
         attr = getMMLType a
 
 makeArray :: TextType -> [Alignment] -> [ArrayLine] -> Element
@@ -113,12 +122,13 @@ makeArray tt as ls = unode "mtable" $
          setAlignment AlignCenter  = withAttribute "columnalign" "center"
          as'                       = as ++ cycle [AlignCenter]
 
-withAttribute :: String -> String -> Element -> Element
-withAttribute a = add_attr . Attr (unqual a)
+-- Kept as String for Text.XML.Light
+withAttribute :: String -> T.Text -> Element -> Element
+withAttribute a = add_attr . Attr (unqual a) . T.unpack
 
-accent :: String -> Element
+accent :: T.Text -> Element
 accent = add_attr (Attr (unqual "accent") "true") .
-           unode "mo"
+           tunode "mo"
 
 makeFence :: FormType -> Element -> Element
 makeFence (fromForm -> t) = withAttribute "stretchy" "false" . withAttribute "form" t
@@ -132,25 +142,25 @@ showExp' tt e =
                            getMathMLOperator x FPostfix of
                              Just True -> "true"
                              _         -> "false"
-      in  withAttribute "accent" isaccent $ unode "mo" x
+      in  withAttribute "accent" isaccent $ tunode "mo" x
     _                -> showExp tt e
 
 showExp :: TextType -> Exp -> Element
 showExp tt e =
  case e of
-   ENumber x        -> unode "mn" x
+   ENumber x        -> tunode "mn" x
    EGrouped [x]     -> showExp tt x
    EGrouped xs      -> mrow $ map (showExp tt) xs
    EDelimited start end xs -> mrow $
-                       [ makeStretchy FPrefix (unode "mo" start) | not (null start) ] ++
-                       map (either (makeStretchy FInfix . unode "mo") (showExp tt)) xs ++
-                       [ makeStretchy FPostfix (unode "mo" end) | not (null end) ]
-   EIdentifier x    -> unode "mi" $ toUnicode tt x
-   EMathOperator x  -> unode "mo" x
-   ESymbol Open x   -> makeFence FPrefix $ unode "mo" x
-   ESymbol Close x  -> makeFence FPostfix $ unode "mo" x
-   ESymbol Ord x    -> unode "mi" x
-   ESymbol _ x      -> unode "mo" x
+     [ makeStretchy FPrefix (tunode "mo" start) | not (T.null start) ] ++
+     map (either (makeStretchy FInfix . tunode "mo") (showExp tt)) xs ++
+     [ makeStretchy FPostfix (tunode "mo" end) | not (T.null end) ]
+   EIdentifier x    -> tunode "mi" $ toUnicode tt x
+   EMathOperator x  -> tunode "mo" x
+   ESymbol Open x   -> makeFence FPrefix $ tunode "mo" x
+   ESymbol Close x  -> makeFence FPostfix $ tunode "mo" x
+   ESymbol Ord x    -> tunode "mi" x
+   ESymbol _ x      -> tunode "mo" x
    ESpace x         -> spaceWidth x
    EFraction ft x y -> showFraction tt ft x y
    ESub x y         -> unode "msub" $ map (showExp tt) [x, y]
@@ -169,3 +179,7 @@ showExp tt e =
    EArray as ls     -> makeArray tt as ls
    EText a s        -> makeText a s
    EStyled a es     -> makeStyled a $ map (showExp a) es
+
+-- Kept as String for Text.XML.Light
+tunode :: String -> T.Text -> Element
+tunode s = unode s . T.unpack
